"""JARVIS database models and utilities."""
from bson import ObjectId
from motor.motor_asyncio import AsyncIOMotorClient
from umongo.frameworks import MotorAsyncIOInstance

CLIENT = None
JARVISDB = None
CTC2DB = None
JARVIS_INST = MotorAsyncIOInstance()
CTC2_INST = MotorAsyncIOInstance()


def connect(
    host: str, username: str, password: str, port: int = 27017, testing: bool = False
) -> None:
    """
    Connect to MongoDB.

    Args:
        host: Hostname/IP
        username: Username
        password: Password
        port: Port
    """
    global CLIENT, JARVISDB, CTC2DB, JARVIS_INST, CTC2_INST
    CLIENT = AsyncIOMotorClient(host=host, username=username, password=password, port=port)
    JARVISDB = CLIENT.narvis if testing else CLIENT.jarvis
    CTC2DB = CLIENT.ctc2

    JARVIS_INST.set_db(JARVISDB)
    CTC2_INST.set_db(CTC2DB)


def q(**kwargs: dict) -> dict:
    """uMongo query wrapper."""  # noqa: D403
    query = {}
    for k, v in kwargs.items():
        if k == "_id":
            v = ObjectId(v)
        elif "__" in k:
            k, mod, *_ = k.split("__")
            if mod:
                v = {f"${mod}": v}
        query[k] = v
    return query
